'
'  **************************************************
'  *                                                *
'  *                 CALENDAR.BAS                   *
'  *                                                *
'  *           Prints One Page Calendars            *
'  *             With Holidays in Bold              *
'  *                                                *
'  *             Version 2.3 12-23-88               *
'  *                                                *
'  *      Compile with Microsoft QuickBASIC 4.x     *
'  *                                                *
'  *              Judson D. McClendon               *
'  *              329 37th Court N.E.               *
'  *              Birmingham, AL 35215              *
'  *                 205-853-8440                   *
'  *            Compuserve [74415,1003]             *
'  *                                                *
'  **************************************************
'
'  Declare Functions/Subroutines
'
DECLARE FUNCTION GetNumber% (MaxSize%)
DECLARE FUNCTION GetWeekday% (Month%, Day%, Year%)
DECLARE FUNCTION IsHoliday% (Month%, Day%)
DECLARE SUB AddDays (Days%, Desc$)
DECLARE SUB CalculateHolidays ()
DECLARE SUB Initialize ()
DECLARE SUB MakeHoliday (Day%, DayOfWeek%, Month%, Desc$)
DECLARE SUB PrintCalendar ()
DECLARE SUB StoreHoliday (HMonth%, HDay%, HDesc$)
DECLARE SUB SubtractDays (Days%, Desc$)

DEFINT A-Z

CONST FALSE = 0, TRUE = NOT FALSE
CONST MaxHolidays = 30
CONST Sunday = 0, Monday = 1, Tuesday = 2, Wednesday = 3
CONST Thursday = 4, Friday = 5, Saturday = 6
CONST Jan = 1, Feb = 2, Mar = 3, Apr = 4, May = 5, Jun = 6
CONST Jul = 7, Aug = 8, Sep = 9, Oct = 10, Nov = 11, Dec = 12
CONST FirstWeek = 1, SecondWeek = 8, ThirdWeek = 15, FourthWeek = 22
CONST LastWeek30 = 24, LastWeek31 = 25
CONST BeginText = 21

'
'  Month Table
'
TYPE MonthDef
   LName    AS STRING * 19
   SName    AS STRING * 3
   MaxDays  AS INTEGER
   DayOf    AS INTEGER
   Weekday  AS INTEGER
END TYPE

DIM SHARED MonthTab(1 TO 12) AS MonthDef

DATA "   J A N U A R Y   ", Jan, 31
DATA "  F E B R U A R Y  ", Feb, 28
DATA "     M A R C H     ", Mar, 31
DATA "     A P R I L     ", Apr, 30
DATA "       M A Y       ", May, 31
DATA "      J U N E      ", Jun, 30
DATA "      J U L Y      ", Jul, 31
DATA "    A U G U S T    ", Aug, 31
DATA " S E P T E M B E R ", Sep, 30
DATA "   O C T O B E R   ", Oct, 31
DATA "  N O V E M B E R  ", Nov, 30
DATA "  D E C E M B E R  ", Dec, 31


'
'  Paschal Full Moon Table (for Easter calculation)
'
TYPE PaschalDef
   Month    AS INTEGER
   Day      AS INTEGER
END TYPE

DIM SHARED PaschalTab(0 TO 18) AS PaschalDef

'        0     1     2     3     4     5     6     7     8     9
DATA  4,14, 4,03, 3,23, 4,11, 3,31, 4,18, 4,08, 3,28, 4,16, 4,05
DATA  3,25, 4,13, 4,02, 3,22, 4,10, 3,30, 4,17, 4,07, 3,27


'
'  Holiday Table
'
TYPE HolidayDef
   CDate    AS INTEGER
   Month    AS INTEGER
   Day      AS INTEGER
   Desc     AS STRING * 18
END TYPE

DIM SHARED HolidayTab(0 TO MaxHolidays) AS HolidayDef


'
'  Miscellaneous Defines
'
DIM SHARED Month AS INTEGER, Day AS INTEGER, Year AS INTEGER
DIM SHARED Copy AS INTEGER, Copies AS INTEGER, FileName AS STRING
DIM SHARED Holidays AS INTEGER
DIM SHARED WhereTo AS STRING * 1, Printer AS STRING * 1
DIM SHARED BoldOn AS STRING, BoldOff AS STRING, Legend1 AS STRING, Legend2 AS STRING


'
'  Module Level Code
'
CALL Initialize
     
OPEN FileName FOR OUTPUT AS #1
WIDTH #1, 255

CALL CalculateHolidays

FOR Copy = 1 TO Copies
   CALL PrintCalendar
NEXT Copy

CLOSE #1

END

   '
   '  **************************************************
   '  *                                                *
   '  *             Add Days to Month, Day             *
   '  *                                                *
   '  **************************************************
   '
SUB AddDays (Days AS INTEGER, Desc AS STRING)

   Day = Day + Days
   WHILE (Day > MonthTab(Month).MaxDays)
      Day = Day - MonthTab(Month).MaxDays
      Month = Month + 1
   WEND

   CALL StoreHoliday(Month, Day, Desc)

END SUB

   '
   '  **************************************************
   '  *                                                *
   '  *             Set up Holiday Table               *
   '  *                                                *
   '  **************************************************
   '
SUB CalculateHolidays

   Holidays = 0               ' No Holidays loaded

   HolidayTab(0).CDate = 0    ' Load entry 0 with dummy for insert

   '
   '  Holidays With Fixed Dates
   '
   CALL StoreHoliday(Jan, 1, "New Years Day")
   CALL StoreHoliday(Feb, 2, "Ground Hog Day")
   CALL StoreHoliday(Feb, 12, "Lincoln")
   CALL StoreHoliday(Feb, 14, "St. Valentine")
   CALL StoreHoliday(Mar, 17, "St. Patrick")
   CALL StoreHoliday(Apr, 1, "All Fools Day")
   CALL StoreHoliday(Jun, 14, "Flag Day")
   CALL StoreHoliday(Jul, 4, "Independence Day")
   CALL StoreHoliday(Oct, 31, "Halloween")
   CALL StoreHoliday(Nov, 11, "Veterans Day")
   CALL StoreHoliday(Dec, 25, "Christmas")
  
   '
   '  Holidays Occuring on Specific Weeks
   '
   CALL MakeHoliday(ThirdWeek, Monday, Jan, "M.L. King")
   CALL MakeHoliday(ThirdWeek, Monday, Feb, "Washington")
   CALL MakeHoliday(SecondWeek, Sunday, May, "Mother's Day")
   CALL MakeHoliday(LastWeek31, Monday, May, "Memorial Day")
   CALL MakeHoliday(ThirdWeek, Sunday, Jun, "Father's Day")
   CALL MakeHoliday(FirstWeek, Monday, Sep, "Labor Day")
   CALL MakeHoliday(SecondWeek, Monday, Oct, "Columbus Day")
   CALL MakeHoliday(SecondWeek, Tuesday, Nov, "Election Day")
   IF (Year < 1941) THEN
      CALL MakeHoliday(LastWeek30, Thursday, Nov, "Thanksgiving")
   ELSE
      CALL MakeHoliday(FourthWeek, Thursday, Nov, "Thanksgiving")
   END IF

   '
   '        ***  Easter Related Holidays  ***
   '
   '
   '  Easter
   '
   I = Year MOD 19
   Month = PaschalTab(I).Month
   Day = PaschalTab(I).Day
   Weekday = GetWeekday%(Month, Day, Year)
   Days = 7 - Weekday
   CALL AddDays(Days, "Easter")

   '
   '  Good Friday
   '
   CALL SubtractDays(2, "Good Friday")

   '
   '  Palm Sunday
   '
   CALL SubtractDays(5, "Palm Sunday")

   '
   '  Ash Wednesday
   '
   CALL SubtractDays(39, "Ash Wednesday")

   '
   '  Pentecost
   '
   CALL AddDays(95, "Pentecost")

END SUB

   '
   '  **************************************************
   '  *                                                *
   '  *   Accept a Number of MaxSize Digits from KBD   *
   '  *                                                *
   '  **************************************************
   '
FUNCTION GetNumber% (MaxSize AS INTEGER)

   DIM Row AS INTEGER, Col AS INTEGER, Char AS STRING
   DIM Size AS INTEGER, Number AS INTEGER
   
   Row = CSRLIN
   Col = POS(0)
   Size = 0
   Number = 0

   DO
      DO
         Char = INKEY$
      LOOP WHILE (Char = "")

      IF (Char >= "0" AND Char <= "9" AND Size < MaxSize) THEN
         IF (Size > 0 OR Char <> "0") THEN
            Number = Number * 10 + VAL(Char)
            Size = Size + 1
            PRINT Char;
            Col = Col + 1
         END IF
      ELSEIF (Char = CHR$(8) AND Size > 0) THEN
         Number = INT(Number / 10)
         Size = Size - 1
         Col = Col - 1
         LOCATE Row, Col
         PRINT " ";
         LOCATE Row, Col
      END IF
   LOOP WHILE (Char <> CHR$(13))

   IF (Number = 0) THEN
      PRINT "0";
   END IF
   GetNumber% = Number

END FUNCTION

   '
   '  **************************************************
   '  *                                                *
   '  *   Calc Day of Week using Zeller's Congruence   *
   '  *                                                *
   '  **************************************************
   '
FUNCTION GetWeekday% (Month AS INTEGER, Day AS INTEGER, Year AS INTEGER)
  
   DIM M AS LONG, D AS LONG, Y AS LONG  ' Need extra precision

   IF Month < Mar THEN  ' Treat Jan & Feb as 13th & 14th month of prev year
      M = Month + 12
      D = Day
      Y = Year - 1
   ELSE
      M = Month
      D = Day
      Y = Year
   END IF
     
   GetWeekday% = (D + M * 2 + INT((M + 1) * .6) + Y + Y \ 4 - Y \ 100 + Y \ 400 + 1) MOD 7

END FUNCTION

   '
   '  **************************************************
   '  *                                                *
   '  *    Initialize tables & get specs from user     *
   '  *                                                *
   '  **************************************************
   '
SUB Initialize

   CLS
   PRINT TAB(BeginText); "ͻ"
   PRINT TAB(BeginText); "                              "
   PRINT TAB(BeginText); "          CALENDAR            "
   PRINT TAB(BeginText); "                              "
   PRINT TAB(BeginText); "         Version 2.2          "
   PRINT TAB(BeginText); "                              "
   PRINT TAB(BeginText); "     Judson D. McClendon      "
   PRINT TAB(BeginText); "     329 37th Court N.E.      "
   PRINT TAB(BeginText); "     Birmingham, AL 35215     "
   PRINT TAB(BeginText); "        205-853-8440          "
   PRINT TAB(BeginText); "    Compuserve [74415,1003]   "
   PRINT TAB(BeginText); "                              "
   PRINT TAB(BeginText); "ͼ"
   PRINT

   FOR Month = 1 TO 12
      READ MonthTab(Month).LName
      READ MonthTab(Month).SName
      READ MonthTab(Month).MaxDays
   NEXT Month

   FOR I = 0 TO 18
      READ PaschalTab(I).Month
      READ PaschalTab(I).Day
   NEXT I

   DO
      LOCATE 15, BeginText + 23: PRINT SPC(10);  ' Clear previous answer
      LOCATE 15, BeginText: PRINT "Enter year since 1582: ";
      Year = GetNumber%(4)
     
      IF (Year >= 0) AND (Year < 100) THEN   ' Default to 20th century
         Year = Year + 1900
         LOCATE 15, BeginText + 22
         PRINT Year; SPC(10);
      END IF
   LOOP UNTIL (Year > 1582)

   IF (Year MOD 400 = 0) OR ((Year MOD 4 = 0) AND (Year MOD 100 <> 0)) THEN
      MonthTab(2).MaxDays = 29
   END IF

   LOCATE 17, BeginText: PRINT "Write to Printer or Disk (P/D): ";
   DO
      WhereTo = UCASE$(INKEY$)
   LOOP UNTIL (INSTR("PD", WhereTo))
   PRINT WhereTo;

   IF WhereTo = "D" THEN
      FileName = LTRIM$(RTRIM$(STR$(Year))) + ".CAL"
      Copies = 1
      LOCATE 19, BeginText: PRINT "Creating file "; FileName;
   ELSE
      FileName = "LPT1:"
      LOCATE 19, BeginText: PRINT "IBM/Epson/Other compatible (I/E/O): ";
      DO
         Printer = UCASE$(INKEY$)
      LOOP UNTIL (INSTR("IEO", Printer))
      PRINT Printer;
     
      DO
         LOCATE 21, BeginText + 16: PRINT SPC(10);  ' Clear Previous Answer
         LOCATE 21, BeginText: PRINT "How many copies? ";
         Copies = GetNumber%(2)
      LOOP UNTIL (Copies >= 0 AND Copies < 100)
   END IF

   Y$ = LTRIM$(RTRIM$(STR$(Year)))
  
   IF (WhereTo = "P") THEN
      IF (Printer = "I") THEN
         BoldOn = "" + CHR$(8) + CHR$(8)
         BoldOff = ""
         Legend1 = SPACE$(39 - 2 * LEN(Y$)) + CHR$(27) + "[@" + CHR$(4) + CHR$(0) + CHR$(0) + CHR$(0) + CHR$(34) + CHR$(2)
         Legend2 = CHR$(27) + "[@" + CHR$(4) + CHR$(0) + CHR$(0) + CHR$(0) + CHR$(17) + CHR$(1)
      ELSEIF (Printer = "E") THEN
         BoldOn = CHR$(27) + "E" + CHR$(27) + "G"
         BoldOff = CHR$(27) + "F" + CHR$(27) + "H"
         Legend1 = SPACE$(39 - 2 * LEN(Y$)) + CHR$(14)
         Legend2 = ""
      ELSE
         BoldOn = ""
         BoldOff = ""
         Legend1 = SPACE$(39 - LEN(Y$))
         Legend2 = ""
      END IF
   ELSE
      BoldOn = ""
      BoldOff = ""
      Legend1 = SPACE$(39 - LEN(Y$))
      Legend2 = ""
   END IF

   FOR I = 1 TO LEN(Y$)
      Legend1 = Legend1 + MID$(Y$, I, 1) + " "
   NEXT I

END SUB

   '
   '  **************************************************
   '  *                                                *
   '  *      If Date in Holiday table return TRUE      *
   '  *                                                *
   '  **************************************************
   '
FUNCTION IsHoliday% (Month AS INTEGER, Day AS INTEGER)

   CDate = Month * 100 + Day     ' Composit Date for comparison
   Lo = 1: Hi = Holidays         ' Set Table Limits
  
   DO                            ' Binary search Holiday table for CDate
      Mid = (Lo + Hi) \ 2
      SELECT CASE (HolidayTab(Mid).CDate)
         CASE IS < CDate
            Lo = Mid + 1
         CASE IS > CDate
            Hi = Mid - 1
         CASE ELSE
            IsHoliday% = TRUE
            EXIT FUNCTION
      END SELECT
   LOOP UNTIL (Lo > Hi)

   IsHoliday% = FALSE

END FUNCTION

   '
   '  **************************************************
   '  *                                                *
   '  *     Create Holiday for Nth Weekday of month    *
   '  *     and store it in Holiday table              *
   '  *                                                *
   '  **************************************************
   '
SUB MakeHoliday (Day AS INTEGER, DayOfWeek AS INTEGER, Month AS INTEGER, Desc AS STRING)

   Weekday = GetWeekday%(Month, Day, Year)
   Day = Day + ((DayOfWeek + 7 - Weekday) MOD 7)
   CALL StoreHoliday(Month, Day, Desc)

END SUB

   '
   '  **************************************************
   '  *                                                *
   '  *        Print Calendar & Holiday table          *
   '  *                                                *
   '  **************************************************
   '
SUB PrintCalendar

   IF WhereTo = "P" THEN
      LOCATE 23, BeginText: PRINT "Printing copy:"; Copy
   END IF

   PRINT #1, ""
   PRINT #1, Legend1
   PRINT #1, Legend2
   FOR WorkMonth = 1 TO 10 STEP 3
      FOR Month = WorkMonth TO WorkMonth + 2
        PRINT #1, TAB((Month - WorkMonth) * 24 + 6); MonthTab(Month).LName;
      NEXT Month
      PRINT #1, "": PRINT #1, ""
      FOR Month = WorkMonth TO WorkMonth + 2
         PRINT #1, TAB((Month - WorkMonth) * 24 + 6); "SU MO TU WE TH FR SA";
         Day = 1
         MonthTab(Month).Weekday = GetWeekday%(Month, Day, Year)
         MonthTab(Month).DayOf = 1
      NEXT Month
      PRINT #1, ""
      FOR Week = 1 TO 6
         TabCol = 1
         FOR Month = WorkMonth TO WorkMonth + 2
            WHILE MonthTab(Month).DayOf <= MonthTab(Month).MaxDays AND MonthTab(Month).Weekday < 7
               TabSize = (Month - WorkMonth) * 24 + MonthTab(Month).Weekday * 3 + 6 - TabCol
               PRINT #1, SPC(TabSize);
               TabCol = TabCol + TabSize
               IF IsHoliday%(Month, MonthTab(Month).DayOf) THEN
                  PRINT #1, USING "&##&"; BoldOn; MonthTab(Month).DayOf; BoldOff;
               ELSE
                  PRINT #1, USING "##"; MonthTab(Month).DayOf;
               END IF
               TabCol = TabCol + 2
               MonthTab(Month).DayOf = MonthTab(Month).DayOf + 1
               MonthTab(Month).Weekday = MonthTab(Month).Weekday + 1
            WEND
            IF MonthTab(Month).Weekday > 6 THEN MonthTab(Month).Weekday = 0
         NEXT Month
         PRINT #1, ""
      NEXT Week
      PRINT #1, "": PRINT #1, "": PRINT #1, ""
   NEXT WorkMonth

   Stp = (Holidays + 2) \ 3
   FOR I = 1 TO Stp
      FOR J = I TO Holidays STEP Stp
         Month = HolidayTab(J).Month
         Day = HolidayTab(J).Day
         Desc$ = HolidayTab(J).Desc
         PRINT #1, USING " & ## &"; MonthTab(Month).SName; Day; Desc$;
      NEXT J
      PRINT #1, ""
   NEXT I

   PRINT #1, CHR$(12);

END SUB

   '
   '  **************************************************
   '  *                                                *
   '  *   Store Holiday in Holiday table in sequence   *
   '  *   by Month & Day.  CDate field is a composit   *
   '  *   of Month and Day for easier comparison.      *
   '  *                                                *
   '  **************************************************
   '
SUB StoreHoliday (HMonth AS INTEGER, HDay AS INTEGER, HDesc AS STRING)

   CDate = HMonth * 100 + HDay            ' Composit Date for Comparison
  
   I = Holidays                           ' Move Later Holidays up in Table
   WHILE (HolidayTab(I).CDate > CDate)
      HolidayTab(I + 1) = HolidayTab(I)
      I = I - 1
   WEND

   I = I + 1                              ' Insert Holiday in Date Sequence
   HolidayTab(I).CDate = CDate
   HolidayTab(I).Month = HMonth
   HolidayTab(I).Day = HDay
   HolidayTab(I).Desc = HDesc
  
   Holidays = Holidays + 1                ' How Many Holidays in Table

END SUB

   '
   '  **************************************************
   '  *                                                *
   '  *         Subtract Days from Month, Day          *
   '  *                                                *
   '  **************************************************
   '
SUB SubtractDays (Days AS INTEGER, Desc AS STRING)
  
   Day = Day - Days
   WHILE (Day < 1)
      Month = Month - 1
      Day = Day + MonthTab(Month).MaxDays
   WEND

   CALL StoreHoliday(Month, Day, Desc)

END SUB

